#include "SimpleAnalysisFramework/AnalysisClass.h"


//#include "PMGTools/PMGTruthWeightTool.h"
DefineAnalysis(monoHiggstautau)

void monoHiggstautau::Init() {
  // Define signal/control regions
  addRegions({"SR_high", 
              "SR_high_400_750", 
              "SR_high_750"});

  addRegions({"SR_low", 
              "SR_low_100_250", 
              "SR_low_250_400", 
              "SR_low_400_550", 
              "SR_low_550"});
}

void monoHiggstautau::ProcessEvent(AnalysisEvent *event) {

  //auto filterEff = PMGCrossSectionTool->getFilterEff(event->getMCNumber);

  // Retrieve basic object lists
  // PLEASE NOTE UNITS ARE ALL IN GEV, AND NOT ATLAS STANDARD MEV!!!
  auto metVec = event->getMET();
  double met = metVec.Et();
  double met_phi = metVec.Phi();

  double weight = 1.; 

  auto electrons = event->getElectrons(10, 2.47, ELooseBLLH);
  auto muons = event->getMuons(10, 2.7, MuMedium);
  auto candJets = event->getJets(
      20., 2.8); // B-jets not used in OR so don't define them here
  auto taus = event->getTaus(20, 2.5, TauRNNVeryLoose);

  // Overal Removal, should be more or less standard, but with taus included.
  // B-jets treated as normal jets, no special treatment.
  taus = overlapRemoval(taus, electrons, 0.2);
  taus = overlapRemoval(taus, muons, 0.2);
  muons = overlapRemoval(muons, electrons, 0.01, NOT(MuCaloTaggedOnly));
  electrons = overlapRemoval(electrons, muons, 0.01);
  candJets = overlapRemoval(candJets, electrons, 0.2);
  electrons = overlapRemoval(electrons, candJets, 0.4);
  candJets = overlapRemoval(candJets, muons, 0.2);
  muons = overlapRemoval(muons, candJets, 0.4);
  candJets = overlapRemoval(candJets, taus, 0.2);

  // Basic filtering by pT, eta and "ID"
  auto signalJets = filterObjects(candJets, 20);
  auto signalElectrons =
      filterObjects(electrons, 25, 2.47, EIsoLoose | ETightLH);
  auto signalMuons = filterObjects(muons, 25, 2.7, MuIsoLoose | MuMedium);
  auto bjets = filterObjects(signalJets, 20., 2.5, BTag77MV2c10);
  auto signalTaus = filterObjects(taus, 20, 2.5, TauRNNVeryLoose);


  float jet_pt = 0;
  float jet_pt2 = 0;
  float bjet_pt = 0;
  float tau_pt = 0;
  float tau_pt2 = 0;
  float tau_eta = 0; 
  float tau_eta2 = 0; 
  float tau_phi = 0; 
  float tau_phi2 = 0; 
  int nBjets = bjets.size();
  int nJets = signalJets.size();
  int nTaus = signalTaus.size();
  int nMuons = signalMuons.size();
  int nEle = signalElectrons.size();


  if (nJets > 0) {
    jet_pt = signalJets[0].Pt();
  }
  if (nJets > 1) {
    jet_pt2 = signalJets[1].Pt();
  }
  if (nBjets > 0) {
    bjet_pt = bjets[0].Pt();
  }

  // Several tile vetoes are applied in the analysis, this directly affects the
  // acceptance so emulate here. NB: this is period speific, if working with a
  // dataset other than 2015-2018 ATLAS data please disable/modify this.

  int LBA52bad = 0;
  int LBC5bad = 0;
  int LBC63bad = 0;
  int EBA03bad = 0;
  int LBA29bad = 0;
  int LBA30bad = 0;
  int LBA32bad = 0;

  float dphijetmet = 10;
  unsigned int jetcounter = 0;
  
  // For all taus that are either closer to met than the closest jet or closer
  // than 0.2 in phi do the same
  for (unsigned int taucounter = 0; taucounter < signalTaus.size(); taucounter++) {
    if (fabs(metVec.DeltaPhi(signalTaus[taucounter])) < 0.2 ||
        fabs(metVec.DeltaPhi(signalTaus[taucounter])) < dphijetmet) {
      if (signalTaus[taucounter].Eta() > (0. - 0.2) &&
          signalTaus[taucounter].Eta() < (0.9 + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 5.05)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBA52bad = 1;
      }
      // LBC5 in 2016
      if (signalTaus[taucounter].Eta() > (0 - 0.2) &&
          signalTaus[taucounter].Eta() < (0.9 + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 0.44)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBC5bad = 1;
      }
      // LBC63 in 2017:
      if (signalTaus[taucounter].Eta() > (-0.9 - 0.2) &&
          signalTaus[taucounter].Eta() < (0. + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() + 0.147)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBC63bad = 1;
      }
      // EBA03 in 2017
      if (signalTaus[taucounter].Eta() > 0.8 &&
          signalTaus[taucounter].Eta() < 1.7 &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 0.245)) <
              9.81e-02) {
        EBA03bad = 1;
      }
      // LBA29 Run numbers 350310 - 352514
      if (signalTaus[taucounter].Eta() > (0. - 0.2) &&
          signalTaus[taucounter].Eta() < (0.9 + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 2.798)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBA29bad = 1;
      }
      // LBA30 Run numbers 350361 - 352514
      if (signalTaus[taucounter].Eta() > (0. - 0.2) &&
          signalTaus[taucounter].Eta() < (0.9 + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 2.896)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBA30bad = 1;
      }
      // LBA32 Run numbers 355261 - 364292
      if (signalTaus[taucounter].Eta() > (0. - 0.2) &&
          signalTaus[taucounter].Eta() < (0.9 + 0.2) &&
          fabs(TVector2::Phi_mpi_pi(signalTaus[taucounter].Phi() - 3.093)) <
              (0.2 + 9.81e-02 / 2.)) {
        LBA32bad = 1;
      }
    }
  }

  // The tile cleaning is only relevant for fractions of dataset so vetoing
  // events based on them doesn't make much sense. Instead a weight is applied
  // based on fraction of 1 - (number of events in the part of the dataset
  // affected)(total number of events in the dataset). Here integrated
  // luminosity is used, if you are using something other than 2015-2018 ATLAS
  // dataset change the numbers accordingly
  double tile_weight = 1;
  if (LBA52bad) {
    tile_weight = tile_weight * (139.1 - 13.1) / 139.1;
  }
  if (LBC5bad) {
    tile_weight = tile_weight * (139.1 - 17.4) / 139.1;
  }
  if (LBC63bad) {
    tile_weight = tile_weight * (139.1 - 44.3) / 139.1;
  }
  if (EBA03bad) {
    tile_weight = tile_weight * (139.1 - 44.3) / 139.1;
  }
  if (LBA29bad) {
    tile_weight = tile_weight * (139.1 - 11.9) / 139.1;
  }
  if (LBA30bad) {
    tile_weight = tile_weight * (139.1 - 11.8) / 139.1;
  }
  if (LBA32bad) {
    tile_weight = tile_weight * (139.1 - 37.6) / 139.1;
  }

  double sumPt=0;
  double sumPx=0;
  double sumPy=0;

  if (signalTaus.size() >= 2) {
    sumPt = signalTaus[0].Pt() + signalTaus[1].Pt() + met;  
    sumPx = signalTaus[0].Px() + signalTaus[1].Px() + metVec.Px();
    sumPy = signalTaus[0].Py() + signalTaus[1].Py() + metVec.Py();
  }

  double mTtot = sqrt((sumPt*sumPt)-(sumPx*sumPx)-(sumPy*sumPy));
  double mt2 = -1;
  double mintheta = -1;
  float dRtautau = -1;
  float mttau1, mttau2;


  double ditauPt = -1;
  double ditaum = -1;
  float tau_1_charge = -1; 
  float tau_2_charge = -1; 
  double pass_HLT_tau30_tau_25_xe50=0; 
  double pass_HLT_tau60_tau_25_xe50=0; 

  auto weights = event->getMCWeights();
  double trigger_platau_weight=1.; 
  if (signalTaus.size() >= 1) {
    tau_1_charge = signalTaus[0].charge();
    tau_pt = signalTaus[0].Pt();
    tau_eta = signalTaus[0].Eta();
    tau_phi = signalTaus[0].Phi();
  }
  if (signalTaus.size() >= 2) {
    ditauPt = (signalTaus[0]+signalTaus[1]).Pt();
    ditaum = (signalTaus[0] + signalTaus[1]).M();
    tau_2_charge = signalTaus[1].charge(); 
    tau_pt2 = signalTaus[1].Pt();
    dRtautau = signalTaus[0].DeltaR(signalTaus[1]); 
    mttau1 = calcMT(signalTaus[0], metVec);
    mttau2 = calcMT(signalTaus[1], metVec);
    tau_eta2 = signalTaus[1].Eta();
    tau_phi2 = signalTaus[1].Phi();    
  }


  // Weight for platau. Assign weight 1 for events passing HLT_tau60_tau_25_xe50 
  // and weight = (mc16a+mc16d-3.3 lumi)/139.1 for events passing 
  // HLT_tau30_tau_25_xe50, but NOT HLT_tau60_tau_25_xe50
  if(tau_pt > 40 && tau_pt < 65) weight*=(36.21+41.0-33.0)/139.1; 

  //higgs ->tautau branching fraction
  weight *= 0.0627; 
  //filter effs handled externally  
  
  bool presel_1 = nTaus == 2 && nMuons == 0 && nEle == 0 && nBjets ==0 && tau_1_charge != tau_2_charge;
  bool presel_2 = met > 150 && tau_pt > 40 && tau_pt2 > 30;

  //SR low inclucive (all bins)
   if(presel_1 && presel_2 && dRtautau<1.9 && dRtautau>0.6 && mTtot>50 && mttau1>50 && mttau2>25 && ditaum > 75 && ditaum < 125 && (mttau1+mttau2)>100){
      accept("SR_low", weight);
  }
  //SR low bin [100, 250] GeV 
  if(presel_1 && presel_2 && dRtautau<1.9 && dRtautau>0.6 && mTtot>50 && mttau1>50 && mttau2>25 && ditaum > 75 && ditaum < 125 && (mttau1+mttau2)>100 && (mttau1+mttau2)<250){
      accept("SR_low_100_250", weight);
  }
  //SR low bin [250, 400] GeV 
  if(presel_1 && presel_2 && dRtautau<1.9 && dRtautau>0.6 && mTtot>50 && mttau1>50 && mttau2>25 && ditaum > 75 && ditaum < 125 && (mttau1+mttau2)>250 && (mttau1+mttau2)<400){
      accept("SR_low_250_400", weight);
  }
  //SR low bin [400, 550] GeV
  if(presel_1 && presel_2 && dRtautau<1.9 && dRtautau>0.6 && mTtot>50 && mttau1>50 && mttau2>25 && ditaum > 75 && ditaum < 125 && (mttau1+mttau2)>400 && (mttau1+mttau2)<550){
      accept("SR_low_400_550", weight);
  }
  //SR low Bin [550, -] GeV
  if(presel_1 && presel_2 && dRtautau<1.9 && dRtautau>0.6 && mTtot>50 && mttau1>50 && mttau2>25 && ditaum > 75 && ditaum < 125 && (mttau1+mttau2)>550){
      accept("SR_low_550", weight);
  }
  //SR high inclucive (all bins)
  if(presel_1 && presel_2 && dRtautau<2 && mTtot>400 && mttau1>50 && mttau2>100 && ditaum>40 && ditaum<125 && (mttau1+mttau2)>400){
      accept("SR_high", weight);
  }
  //SR high bin [400, 750] GeV
  if(presel_1 && presel_2 && dRtautau<2 && mTtot>400 && mttau1>50 && mttau2>100 && ditaum>40 && ditaum<125 && (mttau1+mttau2)>400 && (mttau1+mttau2)<750){
      accept("SR_high_400_750", weight);
  }
  //SR high bin [750, -] GeV 
  if(presel_1 && presel_2 && dRtautau<2 && mTtot>400 && mttau1>50 && mttau2>100 && ditaum>40 && ditaum<125 && (mttau1+mttau2)>750){
      accept("SR_high_750", weight);
  }
  return;
}
